unit vclclxwiz;

// VCL/CLX wizard
// This wizard automatically adds the symbol definition "VCL" or "CLX"
// to a project options, depending on whether the project appears to
// be a CLX project or a VCL project. Neither symbol is added if the
// project does not appear to be a GUI project.
//
// Copyright  2001 Tempest Software, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

interface

uses SysUtils, Classes, ToolsAPI;

type
  TVclClxNotifier = class(TNotifierObject, IOTAIDENotifier)
  private
    function ContainsSymbol(Symbols: string; const Test: string): Boolean;
    function CurrentProject: IOTAProject;
    procedure DefineSymbol(const Symbol: string);
    function IsVcl: Boolean;
    function IsClx: Boolean;
    procedure SetupProjectOptions;
  public
    procedure AfterCompile(Succeeded: Boolean);
    procedure BeforeCompile(const Project: IOTAProject;
      var Cancel: Boolean);
    procedure FileNotification(NotifyCode: TOTAFileNotification;
      const FileName: String; var Cancel: Boolean);
  end;
  TVclClxWizard = class(TNotifierObject, IOTAWizard)
  public
    constructor Create;
    destructor Destroy; override;
    procedure Execute;
    function GetIDString: String;
    function GetName: String;
    function GetState: TWizardState;
  private
    Index: Integer;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterPackageWizard(TVclClxWizard.Create);
end;

// Write a simple log message to the message view.
procedure Log(const Fmt: string; const Args: array of const);
begin
  (BorlandIDEServices as IOTAMessageServices).AddToolMessage('', Format(Fmt, Args), 'vcl/clx', 0, 0);
end;

{ TVclClxWizard }

// Create the wizard, which simply installs an IDE notifier.
constructor TVclClxWizard.Create;
begin
  Index := (BorlandIDEServices as IOTAServices).AddNotifier(TVclClxNotifier.Create)
end;

// Be sure to unregister the notifier.
destructor TVclClxWizard.Destroy;
begin
  (BorlandIDEServices as IOTAServices).RemoveNotifier(Index);
  inherited;
end;

// Stub: Delphi never calls this procedure.
procedure TVclClxWizard.Execute;
begin
end;

function TVclClxWizard.GetIDString: String;
begin
  Result := 'Tempest Software.VCL/CLX project options wizard';
end;

function TVclClxWizard.GetName: String;
begin
  Result := 'VCL/CLX project options wizard';
end;

// Stub: Delphi never calls this procedure.
function TVclClxWizard.GetState: TWizardState;
begin
  Result := [];
end;

{ TVclClxNotifier }

// Stub: We don't care about compilation.
procedure TVclClxNotifier.AfterCompile(Succeeded: Boolean);
begin
end;

// Stub: We don't care about compilation.
procedure TVclClxNotifier.BeforeCompile(const Project: IOTAProject;
  var Cancel: Boolean);
begin
end;

// Return True if Symbols contains the symbol Test. Symbols
// are separated by semicolons.
function TVclClxNotifier.ContainsSymbol(Symbols: string;
  const Test: string): Boolean;
var
  Pos: Integer;
  Sym: string;
begin
  repeat
    Pos := AnsiPos(';', Symbols);
    if Pos = 0 then
      Sym := Symbols
    else
    begin
      Sym := Copy(Symbols, 1, Pos-1);
      Delete(Symbols, 1, Pos);
    end;
    if SameText(Sym, Test) then
    begin
      Result := True;
      Exit;
    end;
  until Pos = 0;
  Result := False;
end;

// Return the current project.
function TVclClxNotifier.CurrentProject: IOTAProject;
var
  I: Integer;
  Group: IOTAProjectGroup;
  Project: IOTAProject;
  Module: IOTAModule;
  Svc: IOTAModuleServices;
begin
  Result := nil;
  Svc := BorlandIDEServices as IOTAModuleServices;
  for I := 0 to Svc.ModuleCount-1 do
  begin
    Module := Svc.Modules[I];
    if Supports(Module, IOTAProjectGroup, Group) then
    begin
      Result := Group.ActiveProject;
      Exit;
    end
    else if Supports(Module, IOTAProject, Project) then
      Result := Project;
  end;
end;

// Add Symbol to the list of definitions in the project options,
// but only if the symbol is not already defined.
procedure TVclClxNotifier.DefineSymbol(const Symbol: string);
resourcestring
  sAlreadyDefined = 'Project already defines %s';
  sAdded = 'Added %s to project options';
var
  Project: IOTAProject;
  Options: IOTAProjectOptions;
  Defines: string;
begin
  Project := CurrentProject;
  if Project <> nil then
  begin
    Options := Project.ProjectOptions;
    Defines := Options.Values['Defines'];
    if ContainsSymbol(Defines, Symbol) then
      Log(sAlreadyDefined, [Symbol])
    else
    begin
      if Length(Defines) > 0 then
        Defines := Defines + ';';
      Defines := Defines + Symbol;
      Options.Values['Defines'] := Defines;
      Project.MarkModified;
      Log(sAdded, [Symbol])
    end;
  end;
end;

// Notifier: setup the options when the active project changes,
// which might be the creation of a new project, or opening an existing project.
// Ignore the other notifications because they aren't important.
procedure TVclClxNotifier.FileNotification(
  NotifyCode: TOTAFileNotification; const FileName: String;
  var Cancel: Boolean);
begin
  if NotifyCode = ofnActiveProjectChanged then
    SetupProjectOptions;
end;

// Return True if the project is a CLX project.
// Detect the CLX project by looking for the QForms unit.
function TVclClxNotifier.IsClx: Boolean;
var
  Project: IOTAProject;
  I: Integer;
  Info: IOTAModuleInfo;
begin
  Project := CurrentProject;
  if Project <> nil then
  begin
    for I := 0 to Project.GetModuleCount-1 do
    begin
      Info := Project.GetModule(I);
      if SameText(Info.Name, 'QForms') then
      begin
        Result := True;
        Exit;
      end;
    end;
  end;
  Result := False
end;

// Return True if the project is a VCL project.
// Detect the VCL project by looking for the Forms unit.
function TVclClxNotifier.IsVcl: Boolean;
var
  Project: IOTAProject;
  I: Integer;
  Info: IOTAModuleInfo;
begin
  Project := CurrentProject;
  if Project <> nil then
  begin
    for I := 0 to Project.GetModuleCount-1 do
    begin
      Info := Project.GetModule(I);
      if SameText(Info.Name, 'Forms') then
      begin
        Result := True;
        Exit;
      end;
    end;
  end;
  Result := False
end;

// Setup the project options, depending on the nature of the
// project and the units it uses.
procedure TVclClxNotifier.SetupProjectOptions;
begin
  if IsVcl then
    DefineSymbol('VCL')
  else if isClx then
    DefineSymbol('CLX');
end;

end.
